@@ -4,7 +4,7 @@ int fgetpos(FILE *restrict f, fpos_t *restrict pos)
 {
 	off_t off = __ftello(f);
 	if (off < 0) return -1;
-	*(off_t *)pos = off;
+	*(long long *)pos = off;
 	return 0;
 }
 
