@@ -7,12 +7,12 @@ size_t fread(void *restrict destv, size_t size, size_t nmemb, FILE *restrict f)
 {
 	unsigned char *dest = destv;
 	size_t len = size*nmemb, l = len, k;
-
-	/* Never touch the file if length is zero.. */
-	if (!l) return 0;
+	if (!size) nmemb = 0;
 
 	FLOCK(f);
 
+	f->mode |= f->mode-1;
+
 	if (f->rend - f->rpos > 0) {
 		/* First exhaust the buffer. */
 		k = MIN(f->rend - f->rpos, l);
@@ -25,7 +25,7 @@ size_t fread(void *restrict destv, size_t size, size_t nmemb, FILE *restrict f)
 	/* Read the remainder directly */
 	for (; l; l-=k, dest+=k) {
 		k = __toread(f) ? 0 : f->read(f, dest, l);
-		if (k+1<=1) {
+		if (!k) {
 			FUNLOCK(f);
 			return (len-l)/size;
 		}
